@@ -75,20 +75,16 @@ CLINE_MATCHER = [
           (\w+)                              # 2: name
         \)\s*\(""", re.VERBOSE),
     # 4: FUNCTION POINTER VARIABLES
-    None,  # in ScanHeaderContent()
+    None,  # in InitScanner()
     # 5-7: ENUMS
     re.compile(
         r"""^\s*enum\s+
         _?(\w+)                              # 1: name
         \s+\{""", re.VERBOSE),
-    re.compile(r'^\s*typedef\s+enum\s+_?(\w+)\s+\1\s*;'),
+    None,  # in InitScanner()
     re.compile(r'^\s*typedef\s+enum'),
     # 8-11: STRUCTS AND UNIONS
-    re.compile(
-        r"""^\s*typedef\s+
-        (struct|union)\s+                    # 1: struct/union
-        _(\w+)                               # 2: name
-        \s+\2\s*;""", re.VERBOSE),
+    None,  # in InitScanner()
     re.compile(r'^\s*(?:struct|union)\s+_(\w+)\s*;'),
     re.compile(
         r"""^\s*
@@ -115,7 +111,7 @@ CLINE_MATCHER = [
         \s*;""", re.VERBOSE),
     re.compile(r'^\s*typedef\s+'),
     # 15: VARIABLES (extern'ed variables)
-    None,  # in ScanHeaderContent()
+    None,  # in InitScanner()
     # 16: VARIABLES
     re.compile(
         r"""^\s*
@@ -130,9 +126,9 @@ CLINE_MATCHER = [
         (FINAL_TYPE|DERIVABLE_TYPE|INTERFACE) # 1: variant
         \s*\(""", re.VERBOSE),
     # 18-21: FUNCTIONS
-    None,  # in ScanHeaderContent()
-    None,  # in ScanHeaderContent()
-    re.compile(r'^\s*([A-Za-z]\w*)\s*\('),
+    None,  # in InitScanner()
+    None,  # in InitScanner()
+    re.compile(r'^\s*\(?([A-Za-z]\w*)\)?\s*\('),
     re.compile(r'^\s*\('),
     # 22-23: STRUCTS
     re.compile(r'^\s*struct\s+_?(\w+)\s*\*'),
@@ -153,11 +149,14 @@ PLINE_MATCHER = [
         """ % RET_TYPE_MODIFIER, re.VERBOSE),
     re.compile(r'^\s*typedef\s*'),
     # 2-4 :FUNCTIONS
-    None,  # in ScanHeaderContent()
-    None,  # in ScanHeaderContent()
-    None,  # in ScanHeaderContent()
+    None,  # in InitScanner()
+    None,  # in InitScanner()
+    None,  # in InitScanner()
 ]
 
+# Matchers for 2nd previous line
+PPLINE_MATCHER = None
+
 # Matchers for sub expressions
 SUB_MATCHER = [
     # 0: STRUCTS AND UNIONS
@@ -168,6 +167,8 @@ SUB_MATCHER = [
 def Run(options):
     logging.info('options: %s', str(options.__dict__))
 
+    InitScanner(options)
+
     if not os.path.isdir(options.output_dir):
         os.mkdir(options.output_dir)
 
@@ -236,16 +237,104 @@ def Run(options):
         open(overrides_file, 'w', encoding='utf-8').close()
 
 
-#
-# Function    : ScanHeaders
-# Description : This scans a directory tree looking for header files.
-#
-# Arguments   : source_dir - the directory to scan.
-#               section_list - a reference to the hashmap of sections.
-#               seen_headers - set to avoid scanning headers twice
-#
+def InitScanner(options):
+    """Apply options to regexps.
+    """
+
+    # avoid generating regex with |'' (matching no string)
+    # TODO(ensonic): keep in sync with ScanHeaderContent()
+    ignore_decorators = ''
+    optional_decorators_regex = ''
+    if options.ignore_decorators:
+        ignore_decorators = '|' + options.ignore_decorators.replace('()', '\(\w*\)')
+        optional_decorators_regex = '(?:\s+(?:%s))?' % ignore_decorators[1:]
+
+    # FUNCTION POINTER VARIABLES
+    CLINE_MATCHER[4] = re.compile(
+        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+        ((?:const\s+|G_CONST_RETURN\s+)?\w+)           # 1: 1st const
+        (\s+const)?\s*                                 # 2: 2nd const
+        (\**)\s*                                       # 3: ptr
+        \(\*\s*
+          (\w+)                                        # 4: name
+        \)\s*\(""" % ignore_decorators, re.VERBOSE)
+
+    CLINE_MATCHER[6] = re.compile(r'^\s*typedef\s+enum\s+_?(\w+)\s+\1%s\s*;' % optional_decorators_regex)
+    CLINE_MATCHER[8] = re.compile(
+        r"""^\s*typedef\s+
+        (struct|union)\s+                    # 1: struct/union
+        _(\w+)\s+\2                          # 2: name
+        %s                                   # 3: optional decorator
+        \s*;""" % optional_decorators_regex, re.VERBOSE)
+    # OTHER TYPEDEFS
+    CLINE_MATCHER[15] = re.compile(
+        r"""^\s*
+        (?:extern|[A-Za-z_]+VAR%s)\s+
+        (?:%s\w+)
+        (?:\s+\*+|\*+|\s)\s*
+        (?:const\s+)*
+        ([A-Za-z]\w*)                                  # 1: name
+        \s*;""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+    # FUNCTIONS
+    CLINE_MATCHER[18] = re.compile(
+        r"""^\s*
+        (?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+        (%s\w+)                                                     # 1: return type
+        ([\s*]+(?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)\s*     # 2: .. cont'
+        (_[A-Za-z]\w*)                                              # 3: name
+        \s*\(""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+    CLINE_MATCHER[19] = re.compile(
+        r"""^\s*
+        (?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+        (%s\w+)                                                     # 1: return type
+        ([\s*]+(?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)\s*     # 2: .. cont'
+        \(?([A-Za-z]\w*)\)?                                         # 3: name
+        \s*\(""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+
+    PLINE_MATCHER[2] = re.compile(
+        r"""^\s*
+        (?:\b(?:extern%s)\s*)*
+        (%s\w+)                                                     # 1: retun type
+        ((?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)              # 2: .. cont'
+        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+
+    PLINE_MATCHER[3] = re.compile(
+        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+        (%s\w+)                                                     # 1: return type
+        ((?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)              # 2: .. cont'
+        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+
+    PLINE_MATCHER[4] = re.compile(
+        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+        (%s\w+)                                                     # 1: return type
+        (\s+\*+|\*+|\s)\s*                                          # 2: ptr?
+        ([A-Za-z]\w*)                                               # 3: symbols
+        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+
+    # Matchers for 2nd previous line
+    global PPLINE_MATCHER
+    PPLINE_MATCHER = [
+        # 0: FUNCTIONS
+        re.compile(
+            r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
+            (
+              (?:const\s+|G_CONST_RETURN\s+|signed\s+|unsigned\s+|struct\s+|union\s+|enum\s+)*
+              \w+
+              (?:\**\s+\**(?:const|G_CONST_RETURN))?
+              (?:\s+|\s*\*+)
+            )\s*$""" % ignore_decorators, re.VERBOSE)
+        ]
+
 
 def ScanHeaders(source_dir, section_list, decl_list, get_types, seen_headers, options):
+    """Scans a directory tree looking for header files.
+
+    Args:
+      source_dir (str): the directory to scan.
+      section_list (dict): map of section to filenames.
+      seen_headers (set): set to avoid scanning headers twice
+    """
+
     logging.info('Scanning source directory: %s', source_dir)
 
     # This array holds any subdirectories found.
@@ -270,18 +359,19 @@ def ScanHeaders(source_dir, section_list, decl_list, get_types, seen_headers, op
                     get_types, seen_headers, options)
 
 
-#
-# Function    : ScanHeader
-# Description : This scans a header file, looking for declarations of
-#                functions, macros, typedefs, structs and unions, which it
-#                outputs to the decl_list.
-# Arguments   : input_file - the header file to scan.
-#               section_list - a map of sections.
-#               decl_list - a list of declarations
-#               seen_headers - set to avoid scanning headers twice
-# Returns     : it adds declarations to the appropriate list.
-#
 def ScanHeader(input_file, section_list, decl_list, get_types, seen_headers, options):
+    """Scan a header file for doc commants.
+
+    Look for doc comments and extract them. Parse each doc comments and the
+    symbol declaration.
+
+    Args:
+      input_file (str): the header file to scan.
+      section_list (dict): a map of section per filename
+      decl_list (list): a list of declarations
+      seen_headers (set): set to avoid scanning headers twice
+    """
+
     # Don't scan headers twice
     canonical_input_file = os.path.realpath(input_file)
     if canonical_input_file in seen_headers:
@@ -313,21 +403,34 @@ def ScanHeader(input_file, section_list, decl_list, get_types, seen_headers, opt
     with open(input_file, 'r', encoding='utf-8') as hdr:
         input_lines = hdr.readlines()
 
-    slist, doc_comments = ScanHeaderContent(input_lines, decl_list, get_types, options)
+    try:
+        slist, doc_comments = ScanHeaderContent(input_lines, decl_list, get_types, options)
+        logging.info("Scanning %s done", input_file)
 
-    logging.info("Scanning %s done", input_file)
+        liststr = SeparateSubSections(slist, doc_comments)
+        if liststr != '':
+            if file_basename not in section_list:
+                section_list[file_basename] = ''
+            section_list[file_basename] += "<SECTION>\n<FILE>%s</FILE>\n%s</SECTION>\n\n" % (file_basename, liststr)
 
-    liststr = SeparateSubSections(slist, doc_comments)
-    if liststr != '':
-        if file_basename not in section_list:
-            section_list[file_basename] = ''
-        section_list[file_basename] += "<SECTION>\n<FILE>%s</FILE>\n%s</SECTION>\n\n" % (file_basename, liststr)
+    except RuntimeError as e:
+        common.LogWarning(input_file, 0, str(e))
 
 
-# Scan the the given content lines.
-# Returns: a list of symbols found and a set of symbols for which we have a
-#          doc-comment
 def ScanHeaderContent(input_lines, decl_list, get_types, options):
+    """Scan the the given content lines.
+
+    Args:
+      input_lines (list):
+      decl_list (list): symbols declarations
+      get_types (list): lst of symbols that have a get_type function
+      options: commandline options
+
+    Returns:
+      list: a list of symbols found and a set of symbols for which we have a
+            doc-comment
+    """
+
     # Holds the resulting list of declarations.
     slist = []
     # Holds the title of the section
@@ -337,7 +440,7 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
     # The type of declaration we are in, e.g. 'function' or 'macro'.
     in_declaration = ''
     # True if we should skip a block.
-    skip_block = 0
+    skip_block = False
     # The current symbol being declared.
     symbol = None
     # Holds the declaration of the current symbol.
@@ -372,63 +475,13 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
     doc_comment = ''
 
     # avoid generating regex with |'' (matching no string)
-    ignore_decorators = ''
+    # TODO(ensonic): keep in sync with InitScanner()
+    # TODO(ensonic): extract the remaining regexps
+    ignore_decorators = ''          # 1 uses
+    optional_decorators_regex = ''  # 4 uses
     if options.ignore_decorators:
-        ignore_decorators = '|' + options.ignore_decorators
-
-    # FUNCTION POINTER VARIABLES
-    CLINE_MATCHER[4] = re.compile(
-        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-        ((?:const\s+|G_CONST_RETURN\s+)?\w+)           # 1: 1st const
-        (\s+const)?\s*                                 # 2: 2nd const
-        (\**)\s*                                       # 3: ptr
-        \(\*\s*
-          (\w+)                                        # 4: name
-        \)\s*\(""" % ignore_decorators, re.VERBOSE)
-    # OTHER TYPEDEFS
-    CLINE_MATCHER[15] = re.compile(
-        r"""^\s*
-        (?:extern|[A-Za-z_]+VAR%s)\s+
-        (?:%s\w+)
-        (?:\s+\*+|\*+|\s)\s*
-        (?:const\s+)*
-        ([A-Za-z]\w*)                                  # 1: name
-        \s*;""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
-    # FUNCTIONS
-    CLINE_MATCHER[18] = re.compile(
-        r"""^\s*
-        (?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-        (%s\w+)                                                     # 1: return type
-        ([\s*]+(?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)\s*     # 2: .. cont'
-        (_[A-Za-z]\w*)                                              # 3: name
-        \s*\(""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
-    CLINE_MATCHER[19] = re.compile(
-        r"""^\s*
-        (?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-        (%s\w+)                                                     # 1: return type
-        ([\s*]+(?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)\s*     # 2: .. cont'
-        ([A-Za-z]\w*)                                               # 3: name
-        \s*\(""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
-
-    PLINE_MATCHER[2] = re.compile(
-        r"""^\s*
-        (?:\b(?:extern%s)\s*)*
-        (%s\w+)                                                     # 1: retun type
-        ((?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)              # 2: .. cont'
-        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
-
-    PLINE_MATCHER[3] = re.compile(
-        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-        (%s\w+)                                                     # 1: return type
-        ((?:\s*(?:\*+|\bconst\b|\bG_CONST_RETURN\b))*)              # 2: .. cont'
-        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
-
-    PLINE_MATCHER[4] = re.compile(
-        r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-        (%s\w+)                                                     # 1: return type
-        (\s+\*+|\*+|\s)\s*                                          # 2: ptr?
-        ([A-Za-z]\w*)                                               # 3: symbols
-        \s*$""" % (ignore_decorators, RET_TYPE_MODIFIER), re.VERBOSE)
+        ignore_decorators = '|' + options.ignore_decorators.replace('()', '\(\w*\)')
+        optional_decorators_regex = '(?:\s+(?:%s))?' % ignore_decorators[1:]
 
     for line in input_lines:
         # If this is a private header, skip it.
@@ -512,21 +565,24 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
 
             cm = [m.match(line) for m in CLINE_MATCHER]
             pm = [m.match(previous_line) for m in PLINE_MATCHER]
+            ppm = [m.match(pre_previous_line) for m in PPLINE_MATCHER]
 
             # MACROS
 
             if cm[0]:
                 symbol = cm[0].group(1)
                 decl = line
-                # We assume all macros which start with '_' are private.
+                # We assume all macros which start with '_' are private, but
+                # we accept '_' itself which is the standard gettext macro.
                 # We also try to skip the first macro if it looks like the
                 # standard #ifndef HEADER_FILE #define HEADER_FILE etc.
                 # And we only want TRUE & FALSE defined in GLib.
                 if not symbol.startswith('_') \
-                    and (not re.search(r'#ifndef\s+' + symbol, previous_line)
-                         or first_macro == 0) \
-                    and ((symbol != 'TRUE' and symbol != 'FALSE')
-                         or options.module == 'glib'):
+                        and (not re.search(r'#ifndef\s+' + symbol, previous_line)
+                             or first_macro == 0) \
+                        and ((symbol != 'TRUE' and symbol != 'FALSE')
+                             or options.module == 'glib') \
+                        or symbol == '_':
                     in_declaration = 'macro'
                     logging.info('Macro: "%s"', symbol)
                 else:
@@ -700,7 +756,7 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                 decl = line[cm[20].end():]
 
                 if is_inline_func(previous_line):
-                    skip_block = 1
+                    skip_block = True
                     if pm[3]:
                         ret_type = format_ret_type(pm[3].group(1), None, pm[3].group(2))
                         logging.info('Function  (3): "%s", Returns: "%s"', symbol, ret_type)
@@ -715,23 +771,14 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
             # on the previous line(s), and the start of the parameters on this.
             elif cm[21]:
                 decl = line[cm[21].end():]
-                ppm = re.search(
-                    r"""^\s*(?:\b(?:extern|static|inline|G_INLINE_FUNC%s)\s*)*
-                    (
-                      (?:const\s+|G_CONST_RETURN\s+|signed\s+|unsigned\s+|struct\s+|union\s+|enum\s+)*
-                      \w+
-                      (?:\**\s+\**(?:const|G_CONST_RETURN))?
-                      (?:\s+|\s*\*+)
-                    )\s*$""" % ignore_decorators, pre_previous_line, re.VERBOSE)
-
                 if pm[4]:
                     ret_type = pm[4].group(1) + ' ' + pm[4].group(2).strip()
                     symbol = pm[4].group(3)
                     in_declaration = 'function'
                     logging.info('Function (5): "%s", Returns: "%s"', symbol, ret_type)
 
-                elif re.search(r'^\s*\w+\s*$', previous_line) and ppm:
-                    ret_type = ppm.group(1)
+                elif re.search(r'^\s*\w+\s*$', previous_line) and ppm[0]:
+                    ret_type = ppm[0].group(1)
                     ret_type = re.sub(r'\s*\n', '', ret_type, flags=re.MULTILINE)
                     in_declaration = 'function'
 
@@ -769,35 +816,19 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                 in_declaration = 'union'
                 logging.info('Union(_): "%s"', symbol)
         else:
-            logging.info('in decl: skip=%s %s', skip_block, line.strip())
-            # If we were already in the middle of a declaration, we simply add
-            # the current line onto the end of it.
-            if skip_block == 0:
-                decl += line
-            else:
-                # Remove all nested pairs of curly braces.
-                brace_remover = r'{[^{]*}'
-                bm = re.search(brace_remover, line)
-                while bm:
-                    line = re.sub(brace_remover, '', line)
-                    bm = re.search(brace_remover, line)
-                # Then hope at most one remains in the line...
-                bm = re.search(r'(.*?){', line)
-                if bm:
-                    if skip_block == 1:
-                        decl += bm.group(1)
-                    skip_block += 1
-                elif '}' in line:
-                    skip_block -= 1
-                    if skip_block == 1:
-                        # this is a hack to detect the end of declaration
-                        decl += ';'
-                        skip_block = 0
-                        logging.info('2: ---')
+            logging.info('in decl %s: skip=%s %s', in_declaration, skip_block, line.strip())
+            decl += line
 
-                else:
-                    if skip_block == 1:
-                        decl += line
+            if skip_block and '{' in decl:
+                (skip_block, decl) = remove_braced_content(decl)
+                logging.info('in decl: skip=%s decl=[%s]', skip_block, decl)
+
+        pre_previous_line = previous_line
+        previous_line = line
+
+        if skip_block:
+            logging.info('skipping, in decl %s, decl=[%s]', in_declaration, decl)
+            continue
 
         if in_declaration == "g-declare":
             dm = re.search(r'\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*\).*$', decl)
@@ -816,9 +847,9 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                         ModuleObjName, deprecated)
                 in_declaration = ''
 
-        # Note that sometimes functions end in ') G_GNUC_PRINTF (2, 3);' or
-        # ') __attribute__ (...);'.
         if in_declaration == 'function':
+            # Note that sometimes functions end in ') G_GNUC_PRINTF (2, 3);' or
+            # ') __attribute__ (...);'.
             regex = r'\)\s*(G_GNUC_.*|.*DEPRECATED.*%s\s*|__attribute__\s*\(.*\)\s*)*;.*$' % ignore_decorators
             pm = re.search(regex, decl, flags=re.MULTILINE)
             if pm:
@@ -844,7 +875,7 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                     internal = 0
                 deprecated_conditional_nest = int(deprecated_conditional_nest)
                 in_declaration = ''
-                skip_block = 0
+                skip_block = False
 
         if in_declaration == 'user_function':
             if re.search(r'\).*$', decl):
@@ -867,24 +898,41 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                     if AddSymbolToList(slist, symbol):
                         decl_list.append('<MACRO>\n<NAME>%s</NAME>\n%s%s</MACRO>\n' % (symbol, deprecated, decl))
                 else:
+                    logging.info('skip internal macro: [%s]', symbol)
                     internal = 0
                 deprecated_conditional_nest = int(deprecated_conditional_nest)
                 in_declaration = ''
+            else:
+                logging.info('skip empty macro: [%s]', symbol)
 
         if in_declaration == 'enum':
-            em = re.search(r'\}\s*(\w+)?;\s*$', decl)
+            # Examples:
+            # "};"
+            # "} MyEnum;"
+            # "} MyEnum DEPRECATED_FOR(NewEnum);"
+            # "} DEPRECATED_FOR(NewEnum);"
+            em = re.search(r'\n\s*\}\s*(?:(\w+)?%s)?;\s*$' % optional_decorators_regex, decl)
             if em:
                 if symbol == '':
                     symbol = em.group(1)
+                # Enums could contain deprecated values and that doesn't mean
+                # the whole enum is deprecated, so they are ignored when setting
+                # deprecated_conditional_nest above. Here we can check if the
+                # _DEPRECATED is between '}' and ';' which would mean the enum
+                # as a whole is deprecated.
+                if re.search(r'\n\s*\}.*_DEPRECATED.*;\s*$', decl):
+                    deprecated = '<DEPRECATED/>\n'
                 if AddSymbolToList(slist, symbol):
-                    decl_list.append('<ENUM>\n<NAME>%s</NAME>\n%s%s</ENUM>\n' % (symbol, deprecated, decl))
+                    stripped_decl = re.sub(optional_decorators_regex, '', decl)
+                    decl_list.append('<ENUM>\n<NAME>%s</NAME>\n%s%s</ENUM>\n' % (symbol, deprecated, stripped_decl))
                 deprecated_conditional_nest = int(deprecated_conditional_nest)
                 in_declaration = ''
 
         # We try to handle nested structs/unions, but unmatched brackets in
         # comments will cause problems.
         if in_declaration == 'struct' or in_declaration == 'union':
-            sm = re.search(r'\n\}\s*(\w*);\s*$', decl)
+            # Same regex as for enum
+            sm = re.search(r'\n\}\s*(?:(\w+)?%s)?;\s*$' % optional_decorators_regex, decl)
             if level <= 1 and sm:
                 if symbol == '':
                     symbol = sm.group(1)
@@ -898,8 +946,9 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                 logging.info('Store struct: "%s"', symbol)
                 if AddSymbolToList(slist, symbol):
                     structsym = in_declaration.upper()
+                    stripped_decl = re.sub('(%s)' % optional_decorators_regex, '', decl)
                     decl_list.append('<%s>\n<NAME>%s</NAME>\n%s%s</%s>\n' %
-                                     (structsym, symbol, deprecated, decl, structsym))
+                                     (structsym, symbol, deprecated, stripped_decl, structsym))
                     if symbol in forward_decls:
                         del forward_decls[symbol]
                 deprecated_conditional_nest = int(deprecated_conditional_nest)
@@ -911,8 +960,9 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
                 level -= line.count('}')
                 logging.info('struct/union level : %d', level)
 
-        pre_previous_line = previous_line
-        previous_line = line
+    # here we want in_declaration=='', otherwise we have a partial declaration
+    if in_declaration != '':
+        raise RuntimeError('partial declaration (%s) : %s ' % (in_declaration, decl))
 
     # print remaining forward declarations
     for symbol in sorted(forward_decls.keys()):
@@ -926,15 +976,45 @@ def ScanHeaderContent(input_lines, decl_list, get_types, options):
     return slist, doc_comments
 
 
+def remove_braced_content(decl):
+    """Remove all nested pairs of curly braces.
+
+    Args:
+      decl (str): the decl
+
+    Returns:
+      str: a declaration stripped of braced content
+    """
+
+    skip_block = True
+    # Remove all nested pairs of curly braces.
+    brace_remover = r'{[^{]*?}'
+    bm = re.search(brace_remover, decl)
+    while bm:
+        decl = re.sub(brace_remover, '', decl)
+        logging.info('decl=[%s]' % decl)
+        bm = re.search(brace_remover, decl)
+
+    # If all '{' have been matched and removed, we're done
+    bm = re.search(r'(.*?){', decl)
+    if not bm:
+        # this is a hack to detect the end of declaration
+        decl = decl.rstrip() + ';'
+        skip_block = False
+        logging.info('skip_block done')
+
+    return skip_block, decl
+
+
 def is_inline_func(line):
     line = line.strip()
     if line.startswith('G_INLINE_FUNC'):
         logging.info('skip block after G_INLINE_FUNC function')
-        return 1
+        return True
     if re.search(r'static\s+inline', line):
         logging.info('skip block after static inline function')
-        return 1
-    return 0
+        return True
+    return False
 
 
 def format_ret_type(base_type, const, ptr):
@@ -946,10 +1026,20 @@ def format_ret_type(base_type, const, ptr):
     return ret_type
 
 
-# Separate the standard macros and functions, placing them at the
-# end of the current section, in a subsection named 'Standard'.
-# do this in a loop to catch object, enums and flags
 def SeparateSubSections(slist, doc_comments):
+    """Separate the standard macros and functions.
+
+    Place them at the end of the current section, in a subsection named
+    'Standard'. Do this in a loop to catch objects, enums and flags.
+
+    Args:
+      slist (list): list of symbols
+      doc_comments (dict): comments for each symbol
+
+    Returns:
+      str: the section doc xml fomatted as string
+    """
+
     klass = lclass = prefix = lprefix = None
     standard_decl = []
     liststr = '\n'.join(s for s in slist if s) + '\n'
