@@ -71,6 +71,7 @@ class ScanHeaderContentTestCase(unittest.TestCase):
             deprecated_guards='GTKDOC_TESTER_DISABLE_DEPRECATED',
             ignore_decorators='',
             rebuild_types=False)
+        scan.InitScanner(self.options)
 
     def scanHeaderContent(self, content):
         return scan.ScanHeaderContent(content, self.decls, self.types,
@@ -220,6 +221,41 @@ class ScanHeaderContentEnum(ScanHeaderContentTestCase):
             header.splitlines(keepends=True))
         self.assertDecl('Data', header, slist)
 
+    def test_HandleDeprecatedMemberDecorator(self):
+        header = textwrap.dedent("""\
+            typedef enum {
+              VAL_DEFAULT,
+              OTHER_VAL MY_DEPRECATED_FOR(VAL_DEFAULT),
+            } Data;""")
+        expected = textwrap.dedent("""\
+            typedef enum {
+              VAL_DEFAULT,
+              OTHER_VAL,
+            } Data;""")
+        self.options.ignore_decorators = 'MY_DEPRECATED_FOR()'
+        scan.InitScanner(self.options)
+        slist, doc_comments = self.scanHeaderContent(
+            header.splitlines(keepends=True))
+        self.assertDecl('Data', expected, slist)
+
+    def test_HandleDeprecatedDecorator(self):
+        header = textwrap.dedent("""\
+            typedef enum {
+              VAL_DEFAULT,
+              OTHER_VAL,
+            } Data MY_DEPRECATED_FOR(OtherEnum);""")
+        expected = textwrap.dedent("""\
+            <DEPRECATED/>
+            typedef enum {
+              VAL_DEFAULT,
+              OTHER_VAL,
+            } Data;""")
+        self.options.ignore_decorators = 'MY_DEPRECATED_FOR()'
+        scan.InitScanner(self.options)
+        slist, doc_comments = self.scanHeaderContent(
+            header.splitlines(keepends=True))
+        self.assertDecl('Data', expected, slist)
+
 
 class ScanHeaderContentFunctions(ScanHeaderContentTestCase):
     """Test parsing of function declarations."""
@@ -242,7 +278,7 @@ class ScanHeaderContentFunctions(ScanHeaderContentTestCase):
         self.assertNoDeclFound(slist)
 
     @parameterized.expand(INLINE_MODIFIERS)
-    def test_IgnoresInternalInlineFunction(self, _, modifier):
+    def test_IgnoresInternalInlineVoidFunction(self, _, modifier):
         header = textwrap.dedent("""\
             %s void _internal(void) {
             }""" % modifier)
@@ -250,6 +286,15 @@ class ScanHeaderContentFunctions(ScanHeaderContentTestCase):
             header.splitlines(keepends=True))
         self.assertNoDeclFound(slist)
 
+    @parameterized.expand(INLINE_MODIFIERS)
+    def test_IgnoresInternalInlineFunction(self, _, modifier):
+        header = textwrap.dedent("""\
+            %s int _internal(int a) {
+              return a + a; }""" % modifier)
+        slist, doc_comments = self.scanHeaderContent(
+            header.splitlines(keepends=True))
+        self.assertNoDeclFound(slist)
+
     @parameterized.expand([(t.replace(' ', '_'), t) for t in BASIC_TYPES_WITH_VOID])
     def test_HandlesReturnValue(self, _, ret_type):
         header = '%s func(void);' % ret_type
@@ -355,6 +400,18 @@ class ScanHeaderContentFunctions(ScanHeaderContentTestCase):
             header.splitlines(keepends=True))
         self.assertDecl('func', 'int', 'int a', slist)
 
+    @parameterized.expand(INLINE_MODIFIERS)
+    def test_FindsInlineFunctionWithParenthesisName(self, _, modifier):
+        header = textwrap.dedent("""\
+            %s void
+            (func) (void)
+            {
+            }
+            """ % modifier)
+        slist, doc_comments = self.scanHeaderContent(
+            header.splitlines(keepends=True))
+        self.assertDecl('func', 'void', 'void', slist)
+
 
 class ScanHeaderContentMacros(ScanHeaderContentTestCase):
     """Test parsing of macro declarations."""
@@ -447,7 +504,7 @@ class ScanHeaderContentStructs(ScanHeaderContentTestCase):
         slist, doc_comments = self.scanHeaderContent([header])
         self.assertNoDeclFound(slist)
 
-    def test_IgnoresPrivteStruct(self):
+    def test_IgnoresPrivateStruct(self):
         header = 'struct _x;'
         slist, doc_comments = self.scanHeaderContent([header])
         self.assertNoDeclFound(slist)
@@ -471,6 +528,30 @@ class ScanHeaderContentStructs(ScanHeaderContentTestCase):
             header.splitlines(keepends=True))
         self.assertIn('<TITLE>GtkdocObject</TITLE>', slist)
 
+    def test_DeprecatedDecorator(self):
+        header = textwrap.dedent("""\
+            typedef struct {
+              int x;
+            } Data MY_DEPRECATED_FOR(OtherStruct);""")
+        expected = textwrap.dedent("""\
+            <DEPRECATED/>
+            typedef struct {
+              int x;
+            } Data;""")
+        self.options.ignore_decorators = 'MY_DEPRECATED_FOR()'
+        scan.InitScanner(self.options)
+        slist, doc_comments = self.scanHeaderContent(
+            header.splitlines(keepends=True))
+        self.assertDecl('Data', expected, slist)
+
+    def test_DeprecatedOpaqueStructTypedef(self):
+        header = 'typedef struct _data data MY_DEPRECATED_FOR(OtherData);'
+        expected = '<DEPRECATED/>\n'
+        self.options.ignore_decorators = 'MY_DEPRECATED_FOR()'
+        scan.InitScanner(self.options)
+        slist, doc_comments = self.scanHeaderContent([header])
+        self.assertDecl('data', expected, slist)
+
 
 class ScanHeaderContentUnions(ScanHeaderContentTestCase):
     """Test parsing of union declarations."""
@@ -506,8 +587,8 @@ class ScanHeaderContentUnions(ScanHeaderContentTestCase):
         slist, doc_comments = self.scanHeaderContent([header])
         self.assertNoDeclFound(slist)
 
-    def test_IgnoresPrivteStruct(self):
-        header = 'union _x x;'
+    def test_IgnoresPrivateUnion(self):
+        header = 'union _x;'
         slist, doc_comments = self.scanHeaderContent([header])
         self.assertNoDeclFound(slist)
 
@@ -654,11 +735,6 @@ class ScanHeaderContentTypedefs(ScanHeaderContentTestCase):
         slist, doc_comments = self.scanHeaderContent([header])
         self.assertDecl('uint', header, slist)
 
-    def test_SkipsEnumTypedefDifferentName(self):
-        header = 'typedef enum data dateenum;'
-        slist, doc_comments = self.scanHeaderContent([header])
-        self.assertNoDeclFound(slist)
-
     def test_SkipsEnumTypedefUnderscoreName(self):
         header = 'typedef enum _data data;'
         slist, doc_comments = self.scanHeaderContent([header])
@@ -761,12 +837,95 @@ class SeparateSubSections(ScanHeaderContentTestCase):
             liststr.splitlines())
 
 
+class RemoveBracedContent(ScanHeaderContentTestCase):
+
+    def test_OneLineFunctionBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int function(int a) { return a + a; }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int function(int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_SimpleFunctionBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int function(int a) {
+              return a + a;
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int function(int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_SimpleFunctionWithNewlineBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int function(int a)
+            {
+              return a + a;
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int function(int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_NestedFunctionBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int function(int a) {
+              if (a > 0) {
+                return a + a;
+              } else {
+                return a - a;
+              }
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int function(int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_NestedFunctionWithNewlinesBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int function(int a)
+            {
+              if (a > 0)
+              {
+                return a + a;
+              }
+              else
+              {
+                return a - a;
+              }
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int function(int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_SimpleFunctionWithParenthesisBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int
+            (function) (int a)
+            {
+              return a + a;
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual("static inline int\n(function) (int a);", decl)
+        self.assertEqual(skip, False)
+
+    def test_FunctionWithMultilineParamsBodyIsRemoved(self):
+        decl = textwrap.dedent("""\
+            static inline int
+            function (int a,
+                      int b)
+            {
+              return a + b;
+            }""")
+        (skip, decl) = scan.remove_braced_content(decl)
+        self.assertEqual(
+            "static inline int\nfunction (int a,\n          int b);", decl)
+        self.assertEqual(skip, False)
+
+
 if __name__ == '__main__':
+    from gtkdoc import common
+    common.setup_logging()
+
     unittest.main()
 
-    # from gtkdoc import common
-    # common.setup_logging()
-    #
-    # t = ScanHeaderContentUserFunction()
+    # t = RemoveBracedContent()
     # t.setUp()
-    # t.test_FindsFunctionStruct_Void_WithLinebreakAfterRetType()
+    # t.test_NestedFunctionBodyIsRemoved()
